/*
 *  MSSQL Communication Library -- Low-Level I/O
 *
 *  (c) 1998 Arno Pedusaar <psaar@fenar.ee>
 *  (c) 1998 Martin Mares <mj@ucw.cz>
 */

/*
 * Every channel type has 4 functions:
 *
 *   int XX_connect(MSSQL *s,char *name,int port);
 *      connects to the data source
 *
 *   int XX_disconnect(MSSQL *s);
 *      closes the link
 *
 *   int XX_read(MSSQL *s,unsigned char *buf,int len);
 *      _guaranteed_ read of len bytes into buf
 *         OK  -> return 0
 *         ERR -> return -1
 *
 *   int XX_write(MSSQL *s,unsigned char *buf,int len);
 *      _guaranteed_ write of len bytes from buf
 *         OK  -> return 0
 *         ERR -> return -1
 *
 */

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <stdio.h>
#include <assert.h>
#include <ctype.h>
#include <errno.h>

#include "mssql.h"

/*
 * INET socket
 */

static int 
SOCKET_connect(MSSQL *s,char *name,int port)
{
struct sockaddr_in serverSa;
struct hostent *he;

	s->fd=socket(AF_INET,SOCK_STREAM,0);
	if (s->fd<0) {
		fprintf(stderr, "socket: %m\n");
		return -1;
	}

	he=gethostbyname(name);
	if (!he) {
		fprintf(stderr, "mssql: Unable to resolve %s: %s\n", name, strerror(errno));
		return -1;
	}
	serverSa.sin_family=AF_INET;
	serverSa.sin_port=htons(port);
	memcpy(&serverSa.sin_addr.s_addr, he->h_addr, 4);

	if (connect(s->fd,(struct sockaddr *)&serverSa,sizeof(serverSa))<0) {
		fprintf(stderr, "mssql: Unable to connect to %s: %s\n", name, strerror(errno));
		return -1;
	}

	return 0;
}

static int
SOCKET_disconnect(MSSQL *s)
{
	close(s->fd);
	s->fd = -1;
	return 0;
}

static int
SOCKET_read(MSSQL *s, unsigned char *buf, int buflen)
{
int n;

	while (buflen) {
		n=read(s->fd,buf,buflen);
		if (n<0) {
			fprintf(stderr, "mssql: Network error: %s\n", strerror(errno));
			return -1;
		} else if (!n) {
			fprintf(stderr, "mssql: Connection closed by the server.\n");
			return -1;
		}
		buf+=n;
		buflen-=n;
	}
	s->bytesIn+=buflen;
	return 0;
}

static int
SOCKET_write(MSSQL *s, unsigned char *buf, int buflen)
{
int n;

	while (buflen) {
		n=write(s->fd,buf,buflen);
		if (n<0) {
			fprintf(stderr, "mssql: Network error: %s\n", strerror(errno));
			return -1;
		}
		buf+=n;
		buflen-=n;
	}
	s->bytesOut+=buflen;
	return 0;
}

/*
 * Set IO channel type
 */

int initIo(MSSQL *s,int method)
{
	switch (method) {
#ifdef HAVE_SSL
	case IO_SSL:
		s->connect=SSL_connect;
		s->disconnect=SSL_disconnect;
		s->read=SSL_read;
		s->write=SSL_write;
		break;
#endif /* HAVE_SSL */
	case IO_SOCKET:
		s->connect=SOCKET_connect;
		s->disconnect=SOCKET_disconnect;
		s->read=SOCKET_read;
		s->write=SOCKET_write;
		break;
	default:
		fprintf(stderr, "mssql: Unknown I/O type!\n");
		return -1;
	}
	return 0;
}
